#!/bin/csh -e
# seqs_chosen_via_median_info.csh
# Arg1 = protein sequence in fasta format
# Arg2 = blastpgp database
# Arg3 = blastpgp processors (-a)

# This program is licensed to you under the Fred
# Hutchinos Cancer Research Center (FHCRC)
# NONCOMMERICAL LICENSE.  A copy of the license may be found at
# http://blocks.fhcrc.org/sift/license.html and should be attached
# to this software

unalias cp || true
unalias rm || true

set seq_database = $2
set median_threshold = 2.75;
if ( $3 == "" ) then
	set blastprocessors = 1
else
	set blastprocessors = $3
endif

# 	Cleanup query sequence.  NCBI stuff is intolerant of format
#	deviations.

set tail = $1:t
set pid = $tail:r
echo tail is $tail
set query = $TMPDIR/$tail.query
echo query is $query

echo "fastaseqs $1 $query     >& /dev/null"
fastaseqs $1 $query	>& /dev/null



# (1) PSIBLAST

set iterations = 2
# the query is called $$.fasta, get pid from that

#######   FILTER ############################
#$BLASTFILTER/seg $query -x > $query.imp
echo "cp $query $query.unfiltered"
cp $query $query.unfiltered
#mv -f $query.imp $query # this has the filtered query

# b option, # of alignments to show (else it is 250)
# m option, type of alignment 0-pairwise, 6-flat master slave w/out identites

# make error file & remove only if PSI-BLAST is finished within time allotted
# not filtering query because Jorja says new statistics take care of that

echo "PSI-BLAST time limit exceeded." > $TMPDIR/$pid.time.error
limit cputime 30m

if ( $?SIFT_DEBUG ) then
	echo "if ! ( -e "/tmp/savethis.out" ) blastpgp -a $blastprocessors -d $seq_database -i $query -o /tmp/savethis.out -m 0 -j $iterations -e .0001 -h 0.002 -b 399"
	      if ! ( -e "/tmp/savethis.out" ) blastpgp -a $blastprocessors -d $seq_database -i $query -o /tmp/savethis.out -m 0 -j $iterations -e .0001 -h 0.002 -b 399;
	echo "cp -a /tmp/savethis.out $query.out"
	cp -a /tmp/savethis.out $query.out
else
					echo "blastpgp -a $blastprocessors -d $seq_database -i $query -o $query.out        -m 0 -j $iterations -e .0001 -h 0.002 -b 399"
					      blastpgp -a $blastprocessors -d $seq_database -i $query -o $query.out        -m 0 -j $iterations -e .0001 -h 0.002 -b 399
endif

if ($status != 0) then
        echo "exiting because stauts not equal to 0"
        exit (-1)
endif
unlimit cputime

# finished within 10 minutes, remove error file
echo "rm -f $TMPDIR/$pid.time.error"
rm -f $TMPDIR/$pid.time.error

##echo done psiblast

# get the psiblast alignment of all the sequences found.

echo "/usr/lib/sift/bin/psiblast_res_to_fasta_dbpairwise $query.out $query.globalX $iterations $query.unfiltered"
      /usr/lib/sift/bin/psiblast_res_to_fasta_dbpairwise $query.out $query.globalX $iterations $query.unfiltered
if ($status != 0) then
	exit (-1)
endif

echo "/usr/lib/sift/bin/clump_output_alignedseq $query.globalX $TMPDIR/$pid.clumped .9 0 >& /dev/null"
      /usr/lib/sift/bin/clump_output_alignedseq $query.globalX $TMPDIR/$pid.clumped .9 0 >& /dev/null

echo "choose seqs time limit exceeded." > $TMPDIR/$pid.time.error
limit cputime 30m

echo "/usr/lib/sift/bin/choose_seqs_via_psiblastseedmedian $query.unfiltered $TMPDIR/$pid.clumped $query.selectedclumped 1.0 $pid $median_threshold"
      /usr/lib/sift/bin/choose_seqs_via_psiblastseedmedian $query.unfiltered $TMPDIR/$pid.clumped $query.selectedclumped 1.0 $pid $median_threshold

if ($status != 0) then
       echo "exiting because stauts not equal to 0"
       exit (-1)
endif
unlimit cputime
rm -f $TMPDIR/$pid.time.error

echo "/usr/lib/sift/bin/consensus_to_seq $query.selectedclumped $TMPDIR/$pid.clumped.consensuskey $TMPDIR/$pid.selected >& /dev/null"
      /usr/lib/sift/bin/consensus_to_seq $query.selectedclumped $TMPDIR/$pid.clumped.consensuskey $TMPDIR/$pid.selected >& /dev/null


echo "/usr/lib/sift/bin/seqs_from_psiblast_res $query.out $TMPDIR/$pid.selected $iterations $query.unfiltered $TMPDIR/$pid.alignedfasta $pid"
      /usr/lib/sift/bin/seqs_from_psiblast_res $query.out $TMPDIR/$pid.selected $iterations $query.unfiltered $TMPDIR/$pid.alignedfasta $pid

echo "rm -f"
	rm -f $TMPDIR/$pid.clumped*
	rm -f $query.selectedclumped*
	rm -f $TMPDIR/$pid.selected
	rm -f $TMPDIR/$pid.unaligned
	rm -f $query.globalX
	rm -f $TMPDIR/$pid.psiblastout
	rm -f $pid.TEMP*
	rm -f $query
	rm -f $query.unfiltered
exit (0)
